@@ -1,30 +1,14 @@
 import sys
 import socket
 import asyncio
-import logging
 import functools
+import urllib.parse
+
+from .common import IPTOS_LOWDELAY, DEFAULT_LIMIT, ConnectionEOFError, ConnectionTimeoutError, log
 
 
 PY_37 = sys.version_info >= (3, 7)
 
-IPTOS_NORMAL = 0x0
-IPTOS_LOWDELAY = 0x10
-IPTOS_THROUGHPUT = 0x08
-IPTOS_RELIABILITY = 0x04
-IPTOS_MINCOST = 0x02
-DEFAULT_LIMIT = 2 ** 20  # 1Mb
-
-
-log = logging.getLogger("sockio")
-
-
-class ConnectionEOFError(ConnectionError):
-    pass
-
-
-class ConnectionTimeoutError(ConnectionError):
-    pass
-
 
 def ensure_connection(f):
     assert asyncio.iscoroutinefunction(f)
@@ -54,7 +38,7 @@ def raw_handle_read(f):
     async def wrapper(self, *args, **kwargs):
         try:
             reply = await f(self, *args, **kwargs)
-        except OSError:
+        except BaseException:
             await self.close()
             raise
         if not reply:
@@ -66,6 +50,7 @@ def raw_handle_read(f):
 
 
 class StreamReaderProtocol(asyncio.StreamReaderProtocol):
+
     def connection_lost(self, exc):
         result = super().connection_lost(exc)
         self._exec_callback("connection_lost_cb", exc)
@@ -89,6 +74,7 @@ class StreamReaderProtocol(asyncio.StreamReaderProtocol):
 
 
 class StreamReader(asyncio.StreamReader):
+
     async def readline(self, eol=b"\n"):
         # This implementation is a copy of the asyncio.StreamReader.readline()
         # with the purpose of supporting different EOL characters.
@@ -114,6 +100,28 @@ class StreamReader(asyncio.StreamReader):
         self._buffer.clear()
 
 
+def configure_socket(sock, no_delay=True, tos=IPTOS_LOWDELAY, keep_alive=None):
+    if hasattr(socket, "TCP_NODELAY") and no_delay:
+        sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
+    if hasattr(socket, "IP_TOS"):
+        sock.setsockopt(socket.SOL_IP, socket.IP_TOS, tos)
+    if keep_alive is not None and hasattr(socket, "SO_KEEPALIVE"):
+        if isinstance(keep_alive, (int, bool)):
+            keep_alive = dict(active=1 if keep_alive in {1, True} else False)
+        active = keep_alive.get('active')
+        idle = keep_alive.get('idle')  # aka keepalive_time
+        interval = keep_alive.get('interval')  # aka keepalive_intvl
+        retry = keep_alive.get('retry')  # aka keepalive_probes
+        if active is not None:
+            sock.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, active)
+        if idle is not None:
+            sock.setsockopt(socket.SOL_TCP, socket.TCP_KEEPIDLE, idle)
+        if interval is not None:
+            sock.setsockopt(socket.SOL_TCP, socket.TCP_KEEPINTVL, interval)
+        if retry is not None:
+            sock.setsockopt(socket.SOL_TCP, socket.TCP_KEEPCNT, retry)
+
+
 async def open_connection(
     host=None,
     port=None,
@@ -137,27 +145,7 @@ async def open_connection(
     )
     writer = asyncio.StreamWriter(transport, protocol, reader, loop)
     sock = writer.transport.get_extra_info("socket")
-    if hasattr(socket, "TCP_NODELAY") and no_delay:
-        sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
-    if hasattr(socket, "IP_TOS"):
-        sock.setsockopt(socket.SOL_IP, socket.IP_TOS, tos)
-    if keep_alive is not None and hasattr(socket, "SO_KEEPALIVE"):
-        if isinstance(keep_alive, (int, bool)):
-            keep_alive = 1 if keep_alive in {1, True} else False
-            sock.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, keep_alive)
-        else:
-            active = keep_alive.get('active')
-            idle = keep_alive.get('idle')  # aka keepalive_time
-            interval = keep_alive.get('interval')  # aka keepalive_intvl
-            retry = keep_alive.get('retry')  # aka keepalive_probes
-            if active is not None:
-                sock.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, active)
-            if idle is not None:
-                sock.setsockopt(socket.SOL_TCP, socket.TCP_KEEPIDLE, idle)
-            if interval is not None:
-                sock.setsockopt(socket.SOL_TCP, socket.TCP_KEEPINTVL, interval)
-            if retry is not None:
-                sock.setsockopt(socket.SOL_TCP, socket.TCP_KEEPCNT, retry)
+    configure_socket(sock, no_delay=no_delay, tos=tos, keep_alive=keep_alive)
     return reader, writer
 
 
@@ -259,7 +247,7 @@ class TCP:
             if loop is not None and not loop.is_closed():
                 self.writer.close()
             else:
-                self._log.warning("could not close stream: loop closed")
+                self._log.info("could not close stream: loop closed")
 
     def __aiter__(self):
         return LineStream(self)
@@ -421,3 +409,13 @@ class TCP:
     def reset_input_buffer(self):
         if self.connected():
             self.reader.reset()
+
+
+def socket_for_url(url, *args, **kwargs):
+    addr = urllib.parse.urlparse(url)
+    scheme = addr.scheme
+    if scheme == "tcp":
+        return TCP(addr.hostname, addr.port, *args, **kwargs)
+    raise ValueError("unsupported async scheme {!r} for {}".format(scheme, url))
+
+
